# renderImage()处理SVG图参阅下面的连接
# https://stackoverflow.com/questions/39093777/renderimage-and-svg-in-shiny-app

shinyServer(function(input, output, session) {
  
  hlmc<- reactive({
    # 调用Python函数识别货劳名称，函数的返回值赋给reactive变量hlmc，这是个字典，R语言自动转换为列表
    parseHlmc(preprocess(input$hlmc))
  })
  
  observe({
    # 控制台输出货劳名称识别结果
    result<- hlmc()
    updateTextAreaInput(session, "hlmc", value = input$hlmc)
  })
  
  output$resultHlmc<- renderText({
    result<- hlmc()
    result[[1]]
  })

  output$segwords<- renderText({
    result<- hlmc()
    gdf<- fromJSON(result[[4]])[[1]]
    paste(gdf$form, collapse=" | ")
  })
  
  output$postags<- renderText({
    result<- hlmc()
    gdf<- fromJSON(result[[4]])[[1]]
    paste(gdf$upos, collapse=" | ")
  })
  
  ## create .svg file of the plot and render it with renderImage()
  
  output$plot_as_svg <- renderImage({
    
    width  <- session$clientData$output_plot_width
    height <- session$clientData$output_plot_height
    mysvgwidth <- width/96
    mysvgheight <- height/96
    
    # 引用货劳名称识别的结果
    result<- hlmc()
    # 用igraph重建语法树
    gdf<- fromJSON(result[[4]])[[1]]
    if (result[[3]] == 0){
      # 语法树
      verts<- gdf[,c(1,2,5)]
      vertsRoot<- data.frame(id=0, form=" ", upos=" ")
      verts<- rbind(verts, vertsRoot)
      names(verts)<-c("order","word","pos")
      verts$name<-as.character(verts$order)
      edges<-gdf[,c(1,3,4)]
      # 建立 igraph图
      g <- graph_from_data_frame(edges, directed=TRUE, vertices=verts)
      V(g)$label<-paste(paste(V(g)$name,V(g)$word,sep=":"),V(g)$pos, sep="\n")
      E(g)$label<-E(g)$deprel
    } else {
      # 语义图
      verts<- gdf[,c(1,2,4)]
      vertsRoot<- data.frame(id=0, form=" ", upos=" ")
      verts<- rbind(verts, vertsRoot)
      names(verts)<-c("order","word","pos")
      verts$name<-as.character(verts$order)
      links<- gdf$deps
      # 空的向量
      source<- target<- relType<-c()
      for (i in 1:length(links)){
        # links2 是一个 n*2的矩阵
        links2<- links[[i]]
        for (j in 1: length(links2[,1])){
          # 第j条边
          link <- links2[j,]
          source[length(source)+1]<- gdf$id[[i]]
          target[length(target)+1]<- link[[1]]
          relType[length(relType)+1]<- link[[2]]
        }
      }
      edges<- data.frame(id=source, head=target, deps=relType)
      edges$head<- as.integer(edges$head)
      # 建立 igraph图
      g <- graph_from_data_frame(edges, directed=TRUE, vertices=verts)
      V(g)$label<-paste(paste(V(g)$name,V(g)$word,sep=":"),V(g)$pos, sep="\n")
      E(g)$label<-E(g)$deps
    }
    # 输出SVG到临时文件
    # This file will be removed later by renderImage
    outfile <- tempfile(fileext='.svg')
    # Cairo要指定中文字体
    CairoFonts(regular="AR PL UKai CN:Book")
    CairoSVG(outfile)
    plot.igraph(g)
    dev.off()
    
    # Return a list containing the filename
    list(src = normalizePath(outfile),
         contentType = 'image/svg+xml',
         width = width,
         height = height,
         alt = "货劳名称识别语法树")
  },
  # SVG图发到客户端后，renderImage删除临时文件
  deleteFile =  TRUE
  )
  
})    